==> jaguar_resty-2.3.3.tar.gz: lib/src/response.dart <==
import 'dart:async';
import 'package:http/http.dart' as http;
import 'expect.dart';
import 'package:async/async.dart';
import 'dart:convert' as codec;

abstract class AsyncResponse<BT> {
  FutureOr<int> get statusCode;

  FutureOr<BT> get body;

  FutureOr<List<int>> get bytes;

  FutureOr<Map<String, String>> get headers;

  FutureOr<bool> get isRedirect;

  FutureOr<bool> get persistentConnection;

  FutureOr<String> get reasonPhrase;

  FutureOr<int> get contentLength;

  FutureOr<http.BaseRequest> get request;

  FutureOr<String> get mimeType;

  FutureOr<String> get encoding;

  AsyncResponse<BT> expect(List<Checker<Response>> conditions);

  AsyncResponse<BT> exact(
      {int statusCode,
      BT body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength});

  /// Runs [func] with [Response] object after request completion
  AsyncResponse<BT> run(dynamic func(Response<BT> resp));
}

class AsyncTResponse<BT> extends DelegatingFuture<Response<BT>>
    implements AsyncResponse<BT> {
  AsyncTResponse(Future<Response<BT>> inner) : super(inner);

  Future<int> get statusCode => then((r) => r.statusCode);

  Future<BT> get body => then((r) => r.body);

  Future<List<int>> get bytes => then((r) => r.bytes);

  Future<Map<String, String>> get headers => then((r) => r.headers);

  Future<bool> get isRedirect => then((r) => r.isRedirect);

  Future<bool> get persistentConnection => then((r) => r.persistentConnection);

  Future<String> get reasonPhrase => then((r) => r.reasonPhrase);

  Future<int> get contentLength => then((r) => r.contentLength);

  Future<http.BaseRequest> get request => then((r) => r.request);

  Future<String> get mimeType => then((r) => r.mimeType);

  Future<String> get encoding => then((r) => r.encoding);

  AsyncTResponse<BT> expect(List<Checker<Response<BT>>> conditions) {
    return new AsyncTResponse<BT>(then((r) => r.expect(conditions)));
  }

  AsyncTResponse<BT> exact(
      {int statusCode,
      BT body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    return new AsyncTResponse<BT>(then((r) {
      r.exact(
          statusCode: statusCode,
          body: body,
          bytes: bytes,
          mimeType: mimeType,
          encoding: encoding,
          headers: headers,
          contentLength: contentLength);
    }));
  }

  /// Runs [func] with [Response] object after request completion
  AsyncTResponse<BT> run(dynamic func(Response<BT> resp)) =>
      new AsyncTResponse<BT>(then((r) async {
        await func(r);
        return r;
      }));
}

class AsyncStringResponse extends DelegatingFuture<StringResponse>
    implements AsyncResponse<String> {
  AsyncStringResponse(Future<Response<String>> inner) : super(inner);

  AsyncStringResponse.from(Future<http.Response> inner)
      : super(inner.then((r) => new StringResponse.from(r)));

  Future<int> get statusCode => then((r) => r.statusCode);

  Future<String> get body => then((r) => r.body);

  Future<List<int>> get bytes => then((r) => r.bytes);

  Future<Map<String, String>> get headers => then((r) => r.headers);

  Future<bool> get isRedirect => then((r) => r.isRedirect);

  Future<bool> get persistentConnection => then((r) => r.persistentConnection);

  Future<String> get reasonPhrase => then((r) => r.reasonPhrase);

  Future<int> get contentLength => then((r) => r.contentLength);

  Future<http.BaseRequest> get request => then((r) => r.request);

  Future<String> get mimeType => then((r) => r.mimeType);

  Future<String> get encoding => then((r) => r.encoding);

  AsyncStringResponse expect(List<Checker<Response<String>>> conditions) {
    return new AsyncStringResponse(then((r) => r.expect(conditions)));
  }

  AsyncStringResponse exact(
      {int statusCode,
      String body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    return new AsyncStringResponse(then((r) => r.exact(
        statusCode: statusCode,
        body: body,
        bytes: bytes,
        mimeType: mimeType,
        encoding: encoding,
        headers: headers,
        contentLength: contentLength)));
  }

  AsyncTResponse<T> json<T, F>([T convert(F d)]) =>
      new AsyncTResponse<T>(then((StringResponse r) => r.json(convert)));

  AsyncTResponse<List<T>> jsonList<T, F>([T convert(F d)]) =>
      new AsyncTResponse<List<T>>(
          then((StringResponse r) => r.jsonList(convert)));

  Future<T> decode<T, F>([T convert(F d)]) => then((r) => r.decode(convert));

  Future<List<T>> decodeList<T, F>([T convert(F d)]) =>
      then((StringResponse r) => r.decodeList(convert));

  /// Runs [func] with [Response] object after request completion
  AsyncStringResponse run(dynamic func(Response<String> resp)) =>
      new AsyncStringResponse(then((r) async {
        await func(r);
        return r;
      }));
}

abstract class Response<T> implements AsyncResponse<T> {
  int get statusCode;

  T get body;

  List<int> get bytes;

  Map<String, String> get headers;

  bool get isRedirect;

  bool get persistentConnection;

  String get reasonPhrase;

  int get contentLength;

  http.BaseRequest get request;

  String get mimeType;

  String get encoding;

  Response<T> expect(List<Checker<Response>> conditions);

  Response<T> exact(
      {int statusCode,
      T body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength});

  Response<T> run(dynamic func(Response<T> resp));
}

class TResponse<T> implements Response<T> {
  final int statusCode;

  final T body;

  final List<int> bytes;

  final Map<String, String> headers;

  final bool isRedirect;

  final bool persistentConnection;

  final String reasonPhrase;

  final int contentLength;

  final http.BaseRequest request;

  final String mimeType;

  final String encoding;

  TResponse(
      {this.statusCode,
      this.body,
      this.bytes,
      this.headers,
      this.isRedirect,
      this.persistentConnection,
      this.reasonPhrase,
      this.contentLength,
      this.request,
      this.mimeType,
      this.encoding});

  TResponse<T> expect(List<Checker<Response<T>>> conditions) {
    final mismatches = conditions
        .map((c) => c(this))
        .reduce((List<Mismatch> v, List<Mismatch> e) => v..addAll(e))
        .toList();
    if (mismatches.length != 0) throw mismatches;
    return this;
  }

  TResponse<T> exact(
      {int statusCode,
      T body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    final conditions = <Checker<Response>>[];
    if (statusCode != null) conditions.add(statusCodeIs(statusCode));
    if (body != null) conditions.add(bodyIs(body));
    if (bytes != null) conditions.add(bodyBytesIs(bytes));
    if (mimeType != null) conditions.add(mimeTypeIs(mimeType));
    if (encoding != null) conditions.add(encodingIs(encoding));
    if (headers != null) {
      headers.forEach(
          (String key, String value) => conditions.add(headersHas(key, value)));
    }
    return expect(conditions);
  }

  TResponse<T> run(dynamic func(Response<T> resp)) {
    func(this);
    return this;
  }
}

class StringResponse implements Response<String> {
  final int statusCode;

  final String body;

  final List<int> bytes;

  final Map<String, String> headers;

  final bool isRedirect;

  final bool persistentConnection;

  final String reasonPhrase;

  final int contentLength;

  final http.BaseRequest request;

  final String mimeType;

  final String encoding;

  StringResponse(
      {this.statusCode,
      this.body,
      this.bytes,
      this.headers,
      this.isRedirect,
      this.persistentConnection,
      this.reasonPhrase,
      this.contentLength,
      this.request,
      this.mimeType,
      this.encoding});

  factory StringResponse.from(http.Response resp) => new StringResponse(
      statusCode: resp.statusCode,
      body: resp.body,
      bytes: resp.bodyBytes,
      headers: resp.headers,
      isRedirect: resp.isRedirect,
      persistentConnection: resp.persistentConnection,
      reasonPhrase: resp.reasonPhrase,
      contentLength: resp.contentLength,
      request: resp.request,
      mimeType: _getMimeType(resp.headers['content-type']),
      encoding: _getEncoding(resp.headers['content-type']));

  TResponse<T> json<T, F>([T convert(F d)]) {
    final d = codec.json.decode(body);
    T ret;
    if (convert != null)
      ret = convert(d);
    else
      ret = d;
    return new TResponse<T>(
      statusCode: statusCode,
      body: ret,
      bytes: bytes,
      headers: headers,
      isRedirect: isRedirect,
      persistentConnection: persistentConnection,
      reasonPhrase: reasonPhrase,
      contentLength: contentLength,
      request: request,
      mimeType: mimeType,
      encoding: encoding,
    );
  }

  TResponse<List<T>> jsonList<T, F>([T convert(F d)]) {
    List d = codec.json.decode(body);
    List<T> ret;
    if (convert != null)
      ret = d.cast<F>().map(convert).cast<T>().toList();
    else
      ret = d;
    return new TResponse<List<T>>(
      statusCode: statusCode,
      body: ret,
      bytes: bytes,
      headers: headers,
      isRedirect: isRedirect,
      persistentConnection: persistentConnection,
      reasonPhrase: reasonPhrase,
      contentLength: contentLength,
      request: request,
      mimeType: mimeType,
      encoding: encoding,
    );
  }

  T decode<T, F>([T convert(F d)]) {
    final d = codec.json.decode(body);
    if (convert == null) return d;
    return convert(d);
  }

  List<T> decodeList<T, F>([T convert(F d)]) {
    List d = codec.json.decode(body);
    if (convert != null) return d.cast<F>().map(convert).cast<T>().toList();
    return d;
  }

  StringResponse expect(List<Checker<Response<String>>> conditions) {
    final mismatches = conditions
        .map((c) => c(this))
        .reduce((List<Mismatch> v, List<Mismatch> e) => v..addAll(e))
        .toList();
    if (mismatches.length != 0) throw mismatches;
    return this;
  }

  StringResponse exact(
      {int statusCode,
      String body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    final conditions = <Checker<Response>>[];
    if (statusCode != null) conditions.add(statusCodeIs(statusCode));
    if (body != null) conditions.add(bodyIs(body));
    if (bytes != null) conditions.add(bodyBytesIs(bytes));
    if (mimeType != null) conditions.add(mimeTypeIs(mimeType));
    if (encoding != null) conditions.add(encodingIs(encoding));
    if (headers != null) {
      headers.forEach(
          (String key, String value) => conditions.add(headersHas(key, value)));
    }
    return expect(conditions);
  }

  StringResponse run(dynamic func(Response<String> resp)) {
    func(this);
    return this;
  }
}

String _getMimeType(String header) {
  header ??= '';
  List<String> parts = header.split(';');
  if (parts.length == 0) return 'text/plain';
  return parts.first.trim();
}

String _getEncoding(String header) {
  header ??= '';
  List<String> parts = header.split(';');
  if (parts.length < 2) return 'utf-8';
  parts.removeAt(0);
  return parts.map((s) {
    List<String> parts = s.split('=');
    if (parts.length != 2) return null;
    if (parts.first != 'charset') return null;
    return parts[1];
  }).firstWhere((e) => e != null, orElse: () => 'utf-8');
}

==> jaguar_resty-2.3.5.tar.gz: lib/src/response.dart <==
import 'dart:async';
import 'package:http/http.dart' as http;
import 'expect.dart';
import 'package:async/async.dart';
import 'dart:convert' as codec;

typedef dynamic After<T>(Response<T> response);

abstract class AsyncResponse<BT> {
  FutureOr<int> get statusCode;

  FutureOr<bool> get isSuccess;

  FutureOr<BT> get body;

  FutureOr<List<int>> get bytes;

  FutureOr<Map<String, String>> get headers;

  FutureOr<bool> get isRedirect;

  FutureOr<bool> get persistentConnection;

  FutureOr<String> get reasonPhrase;

  FutureOr<int> get contentLength;

  FutureOr<http.BaseRequest> get request;

  FutureOr<String> get mimeType;

  FutureOr<String> get encoding;

  AsyncResponse<BT> expect(List<Checker<Response>> conditions);

  AsyncResponse<BT> exact(
      {int statusCode,
      BT body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength});

  /// Runs [func] with [Response] object after request completion
  AsyncResponse<BT> run(dynamic func(Response<BT> resp));

  /// Runs [funcs] with [Response] object after request completion
  AsyncResponse<BT> runAll(List<After<BT>> funcs);
}

class AsyncTResponse<BT> extends DelegatingFuture<Response<BT>>
    implements AsyncResponse<BT> {
  AsyncTResponse(Future<Response<BT>> inner) : super(inner);

  Future<int> get statusCode => then((r) => r.statusCode);

  Future<BT> get body => then((r) => r.body);

  Future<List<int>> get bytes => then((r) => r.bytes);

  Future<Map<String, String>> get headers => then((r) => r.headers);

  Future<bool> get isRedirect => then((r) => r.isRedirect);

  Future<bool> get persistentConnection => then((r) => r.persistentConnection);

  Future<String> get reasonPhrase => then((r) => r.reasonPhrase);

  Future<int> get contentLength => then((r) => r.contentLength);

  Future<http.BaseRequest> get request => then((r) => r.request);

  Future<String> get mimeType => then((r) => r.mimeType);

  Future<String> get encoding => then((r) => r.encoding);

  Future<bool> get isSuccess => then((r) => r.isSuccess);

  AsyncTResponse<BT> expect(List<Checker<Response<BT>>> conditions) {
    return new AsyncTResponse<BT>(then((r) => r.expect(conditions)));
  }

  AsyncTResponse<BT> exact(
      {int statusCode,
      BT body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    return new AsyncTResponse<BT>(then((r) {
      r.exact(
          statusCode: statusCode,
          body: body,
          bytes: bytes,
          mimeType: mimeType,
          encoding: encoding,
          headers: headers,
          contentLength: contentLength);
    }));
  }

  /// Runs [func] with [Response] object after request completion
  AsyncTResponse<BT> run(After<BT> func) =>
      new AsyncTResponse<BT>(then((r) async {
        await func(r);
        return r;
      }));

  /// Runs [funcs] with [Response] object after request completion
  AsyncTResponse<BT> runAll(List<After<BT>> funcs) =>
      new AsyncTResponse<BT>(then((r) async {
        for (After<BT> func in funcs) await func(r);
        return r;
      }));
}

class AsyncStringResponse extends DelegatingFuture<StringResponse>
    implements AsyncResponse<String> {
  AsyncStringResponse(Future<Response<String>> inner) : super(inner);

  AsyncStringResponse.from(Future<http.Response> inner)
      : super(inner.then((r) => new StringResponse.from(r)));

  Future<int> get statusCode => then((r) => r.statusCode);

  Future<String> get body => then((r) => r.body);

  Future<List<int>> get bytes => then((r) => r.bytes);

  Future<Map<String, String>> get headers => then((r) => r.headers);

  Future<bool> get isRedirect => then((r) => r.isRedirect);

  Future<bool> get persistentConnection => then((r) => r.persistentConnection);

  Future<String> get reasonPhrase => then((r) => r.reasonPhrase);

  Future<int> get contentLength => then((r) => r.contentLength);

  Future<http.BaseRequest> get request => then((r) => r.request);

  Future<String> get mimeType => then((r) => r.mimeType);

  Future<String> get encoding => then((r) => r.encoding);

  Future<bool> get isSuccess => then((r) => r.isSuccess);

  AsyncStringResponse expect(List<Checker<Response<String>>> conditions) {
    return new AsyncStringResponse(then((r) => r.expect(conditions)));
  }

  AsyncStringResponse exact(
      {int statusCode,
      String body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    return new AsyncStringResponse(then((r) => r.exact(
        statusCode: statusCode,
        body: body,
        bytes: bytes,
        mimeType: mimeType,
        encoding: encoding,
        headers: headers,
        contentLength: contentLength)));
  }

  AsyncTResponse<T> json<T, F>([T convert(F d)]) =>
      new AsyncTResponse<T>(then((StringResponse r) => r.json(convert)));

  AsyncTResponse<List<T>> jsonList<T, F>([T convert(F d)]) =>
      new AsyncTResponse<List<T>>(
          then((StringResponse r) => r.jsonList(convert)));

  Future<T> decode<T, F>([T convert(F d)]) => then((r) => r.decode(convert));

  Future<List<T>> decodeList<T, F>([T convert(F d)]) =>
      then((StringResponse r) => r.decodeList(convert));

  /// Runs [func] with [Response] object after request completion
  AsyncStringResponse run(After<String> func) =>
      new AsyncStringResponse(then((r) async {
        await func(r);
        return r;
      }));

  /// Runs [funcs] with [Response] object after request completion
  AsyncStringResponse runAll(List<After<String>> funcs) =>
      new AsyncStringResponse(then((r) async {
        for (After<String> func in funcs) await func(r);
        return r;
      }));
}

abstract class Response<T> implements AsyncResponse<T> {
  int get statusCode;

  bool get isSuccess;

  T get body;

  List<int> get bytes;

  Map<String, String> get headers;

  bool get isRedirect;

  bool get persistentConnection;

  String get reasonPhrase;

  int get contentLength;

  http.BaseRequest get request;

  String get mimeType;

  String get encoding;

  Response<T> expect(List<Checker<Response>> conditions);

  Response<T> exact(
      {int statusCode,
      T body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength});

  Response<T> run(After<T> func);

  /// Runs [funcs] with [Response] object after request completion
  Response<T> runAll(List<After<T>> funcs);
}

class TResponse<T> implements Response<T> {
  final int statusCode;

  final T body;

  final List<int> bytes;

  final Map<String, String> headers;

  final bool isRedirect;

  final bool persistentConnection;

  final String reasonPhrase;

  final int contentLength;

  final http.BaseRequest request;

  final String mimeType;

  final String encoding;

  TResponse(
      {this.statusCode,
      this.body,
      this.bytes,
      this.headers,
      this.isRedirect,
      this.persistentConnection,
      this.reasonPhrase,
      this.contentLength,
      this.request,
      this.mimeType,
      this.encoding});

  bool get isSuccess => statusCode >= 200 && statusCode < 300;

  TResponse<T> expect(List<Checker<Response<T>>> conditions) {
    final mismatches = conditions
        .map((c) => c(this))
        .reduce((List<Mismatch> v, List<Mismatch> e) => v..addAll(e))
        .toList();
    if (mismatches.length != 0) throw mismatches;
    return this;
  }

  TResponse<T> exact(
      {int statusCode,
      T body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    final conditions = <Checker<Response>>[];
    if (statusCode != null) conditions.add(statusCodeIs(statusCode));
    if (body != null) conditions.add(bodyIs(body));
    if (bytes != null) conditions.add(bodyBytesIs(bytes));
    if (mimeType != null) conditions.add(mimeTypeIs(mimeType));
    if (encoding != null) conditions.add(encodingIs(encoding));
    if (headers != null) {
      headers.forEach(
          (String key, String value) => conditions.add(headersHas(key, value)));
    }
    return expect(conditions);
  }

  TResponse<T> run(After<T> func) {
    func(this);
    return this;
  }

  /// Runs [funcs] with [Response] object after request completion
  TResponse<T> runAll(List<After<T>> funcs) {
    for (After<T> func in funcs) func(this);
    return this;
  }
}

class StringResponse implements Response<String> {
  final int statusCode;

  final String body;

  final List<int> bytes;

  final Map<String, String> headers;

  final bool isRedirect;

  final bool persistentConnection;

  final String reasonPhrase;

  final int contentLength;

  final http.BaseRequest request;

  final String mimeType;

  final String encoding;

  StringResponse(
      {this.statusCode,
      this.body,
      this.bytes,
      this.headers,
      this.isRedirect,
      this.persistentConnection,
      this.reasonPhrase,
      this.contentLength,
      this.request,
      this.mimeType,
      this.encoding});

  factory StringResponse.from(http.Response resp) => new StringResponse(
      statusCode: resp.statusCode,
      body: resp.body,
      bytes: resp.bodyBytes,
      headers: resp.headers,
      isRedirect: resp.isRedirect,
      persistentConnection: resp.persistentConnection,
      reasonPhrase: resp.reasonPhrase,
      contentLength: resp.contentLength,
      request: resp.request,
      mimeType: _getMimeType(resp.headers['content-type']),
      encoding: _getEncoding(resp.headers['content-type']));

  bool get isSuccess => statusCode >= 200 && statusCode < 300;

  TResponse<T> json<T, F>([T convert(F d)]) {
    final d = codec.json.decode(body);
    T ret;
    if (convert != null)
      ret = convert(d);
    else
      ret = d;
    return new TResponse<T>(
      statusCode: statusCode,
      body: ret,
      bytes: bytes,
      headers: headers,
      isRedirect: isRedirect,
      persistentConnection: persistentConnection,
      reasonPhrase: reasonPhrase,
      contentLength: contentLength,
      request: request,
      mimeType: mimeType,
      encoding: encoding,
    );
  }

  TResponse<List<T>> jsonList<T, F>([T convert(F d)]) {
    List d = codec.json.decode(body);
    List<T> ret;
    if (convert != null)
      ret = d.cast<F>().map(convert).cast<T>().toList();
    else
      ret = d;
    return new TResponse<List<T>>(
      statusCode: statusCode,
      body: ret,
      bytes: bytes,
      headers: headers,
      isRedirect: isRedirect,
      persistentConnection: persistentConnection,
      reasonPhrase: reasonPhrase,
      contentLength: contentLength,
      request: request,
      mimeType: mimeType,
      encoding: encoding,
    );
  }

  T decode<T, F>([T convert(F d)]) {
    final d = codec.json.decode(body);
    if (convert == null) return d;
    return convert(d);
  }

  List<T> decodeList<T, F>([T convert(F d)]) {
    List d = codec.json.decode(body);
    if (convert != null) return d.cast<F>().map(convert).cast<T>().toList();
    return d;
  }

  StringResponse expect(List<Checker<Response<String>>> conditions) {
    final mismatches = conditions
        .map((c) => c(this))
        .reduce((List<Mismatch> v, List<Mismatch> e) => v..addAll(e))
        .toList();
    if (mismatches.length != 0) throw mismatches;
    return this;
  }

  StringResponse exact(
      {int statusCode,
      String body,
      List<int> bytes,
      String mimeType,
      String encoding,
      Map<String, String> headers,
      int contentLength}) {
    final conditions = <Checker<Response>>[];
    if (statusCode != null) conditions.add(statusCodeIs(statusCode));
    if (body != null) conditions.add(bodyIs(body));
    if (bytes != null) conditions.add(bodyBytesIs(bytes));
    if (mimeType != null) conditions.add(mimeTypeIs(mimeType));
    if (encoding != null) conditions.add(encodingIs(encoding));
    if (headers != null) {
      headers.forEach(
          (String key, String value) => conditions.add(headersHas(key, value)));
    }
    return expect(conditions);
  }

  StringResponse run(dynamic func(Response<String> resp)) {
    func(this);
    return this;
  }

  /// Runs [funcs] with [Response] object after request completion
  StringResponse runAll(List<After<String>> funcs) {
    for (After<String> func in funcs) func(this);
    return this;
  }
}

String _getMimeType(String header) {
  header ??= '';
  List<String> parts = header.split(';');
  if (parts.length == 0) return 'text/plain';
  return parts.first.trim();
}

String _getEncoding(String header) {
  header ??= '';
  List<String> parts = header.split(';');
  if (parts.length < 2) return 'utf-8';
  parts.removeAt(0);
  return parts.map((s) {
    List<String> parts = s.split('=');
    if (parts.length != 2) return null;
    if (parts.first != 'charset') return null;
    return parts[1];
  }).firstWhere((e) => e != null, orElse: () => 'utf-8');
}
